"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.request = void 0;
const tslib_1 = require("tslib");
const constants_1 = require("../constants/constants");
const undici_1 = require("undici");
const generate_ds_1 = tslib_1.__importDefault(require("./generate_ds"));
const error_1 = require("./error");
const error_2 = tslib_1.__importDefault(require("../constants/error"));
require("../constants/lang");
const debug_1 = require("./debug");
/**
 * @description Creates a new instance for the request
 */
class HTTPRequest {
    constructor(option) {
        Object.defineProperty(this, "baseURL", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "withUA", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "withDS", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "language", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        this.baseURL = constants_1.Genshin_Battle_API_URL;
        if (option?.route) {
            this.baseURL = option.route;
        }
        this.withUA = option?.withUA || false;
        this.withDS = option?.withDS || false;
        this.language = "en-us" /* Language.EnglishUS */;
    }
    _debug(message) {
        if ((0, debug_1.isDebug)()) {
            console.log(message);
        }
    }
    async get(url, headers, params) {
        const URL = `${this.baseURL}${url}`;
        const requestHeaders = {
            "User-Agent": this.withUA ? constants_1.UA : undefined,
            "x-rpc-language": this.language,
            DS: this.withDS ? (0, generate_ds_1.default)() : undefined,
            ...headers,
        };
        this._debug(`GET ${URL}`);
        this._debug(`Headers: ${JSON.stringify(requestHeaders)}`);
        this._debug(`Params: ${JSON.stringify(params)}`);
        const { body: res, statusCode } = await (0, undici_1.request)(URL, {
            method: "GET",
            headers: requestHeaders,
            query: params,
        });
        if (statusCode !== 200) {
            this._debug(`Request failed with status code ${statusCode}`);
            throw new Error(`Request failed with status code ${statusCode}`);
        }
        const resData = await res.json();
        this._debug(`Retcode: ${resData.retcode}`);
        this._debug(`Message: ${resData.message}`);
        this._debug(`Data: ${JSON.stringify(resData.data)}`);
        if (resData.retcode !== 0) {
            const description = (0, error_2.default)(resData.retcode);
            this._debug(`Error: ${description}`);
            throw new error_1.APIERROR(`${resData.message}`, resData.retcode, description);
        }
        return resData;
    }
    async post(url, headers, data, params) {
        const URL = `${this.baseURL}${url}`;
        const requestHeaders = {
            "User-Agent": this.withUA ? constants_1.UA : undefined,
            "x-rpc-language": this.language,
            DS: this.withDS ? (0, generate_ds_1.default)() : undefined,
            ...headers,
        };
        const body = JSON.stringify(data);
        this._debug(`POST ${URL}`);
        this._debug(`Headers: ${JSON.stringify(requestHeaders)}`);
        this._debug(`Params: ${JSON.stringify(params)}`);
        this._debug(`Body: ${body}`);
        const { body: res, statusCode } = await (0, undici_1.request)(URL, {
            method: "POST",
            headers: requestHeaders,
            body: body,
            query: params,
        });
        if (statusCode !== 200) {
            this._debug(`Request failed with status code ${statusCode}`);
            throw new Error(`Request failed with status code ${statusCode}`);
        }
        const resData = await res.json();
        this._debug(`Retcode: ${resData.retcode}`);
        this._debug(`Message: ${resData.message}`);
        this._debug(`Data: ${JSON.stringify(resData.data)}`);
        if (resData.retcode !== 0 && resData.retcode !== -5003) {
            const description = (0, error_2.default)(resData.retcode);
            this._debug(`Error: ${description}`);
            throw new error_1.APIERROR(`${resData.message}`, resData.retcode, description);
        }
        return resData;
    }
    setLanguage(language) {
        this.language = language;
    }
}
exports.request = HTTPRequest;
